% This simple demo examines if TVAL3 works normally. Please try more demos
% in the "Demos" directory, which would show users what TVAL3 is capable of.
% 
% I: 64x64 phantom (real, two-dimentional)
% A: random matrix without normality and orthogonality (real)
% f: observation with/without noise (real)
%
% Written by: Chengbo Li
% Advisor: Prof. Yin Zhang and Wotao Yin
% CAAM department, Rice University
% 05/21/2009

clear; close all;
path(path,genpath(pwd));
fullscreen = get(0,'ScreenSize');

randn('state',245);
rand('state',245);

% problem size
n = 256;
ratio = .25 ;
p = n; q = n; % p x q is the size of image
m = round(ratio*n^2);

N = n^2;

% sensing matrix
M = round( N / 4 );
omega = randn(N,1);
[temp,omega] = sort(omega);
omega = sort( omega(1:M) );
downsample = @(x) x(omega);
SS.type = '()'; SS.subs{1} = omega; SS.subs{2} = ':';
upsample = @(x) subsasgn( zeros(N,size(x,2)),SS,x);

mat = @(x) reshape(x,n,n);
vec = @(x) x(:);

% take partial 2D DCT measurements.  We have Af(At) = identity (not
% vice-versa of course). Randomly permuted.
rp = randperm(N);
[~,rp_inv] = sort(rp);
rpF = @(x) x(rp);
rp_invF = @(x) x(rp_inv);
        
my_dct2 = @(x) dct(dct(x).').';
my_idct2= @(x) idct(idct(x).').';
Af = @(x) downsample(vec( my_dct2( mat(rpF(mat(x) )) ) ) );
At = @(y) vec( rp_invF( my_idct2( mat( upsample( y ) ) ) ) );

A = linop_handles([m,N], Af, At );

% original image
I = phantom(n);
nrmI = norm(I,'fro');
figure('Name','TVAL3','Position',...
    [fullscreen(1) fullscreen(2) fullscreen(3) fullscreen(4)]);
subplot(131); imshow(I,[]);
title('Original phantom','fontsize',18); drawnow;

% observation
%f = A*I(:);
%favg = mean(abs(f));

myAwgn = @(x,snr) x + ...
        10^( (10*log10(sum(abs(x(:)).^2)/length(x(:))) - snr)/20 )*randn(size(x));
    
snr = 10;  % SNR in dB
x_noisy = myAwgn( I(:), snr );
% add noise
f = A(x_noisy,1);

%% Run TVAL3
clear opts
opts.mu = 2^8;
opts.beta = 2^5;
opts.tol = 1E-3;
opts.maxit = 1000;
opts.TVnorm = 1;
opts.TVAL2=true;
opts.nonneg = false;

t = cputime;
[U, out] = TVAL3(A,f,p,q,opts);
t = cputime - t;

subplot(132); 
imshow(U,[]);
title('Recovered by TVAL3','fontsize',18);
xlabel(sprintf(' %2d%% measurements \n Rel-Err: %4.2f%%, CPU: %4.2fs ',ratio*100,norm(U-I,'fro')/nrmI*100,t),'fontsize',16);

ApdNCG  = A_operator(@(w) A(w,1), @(w) A(w,2));

optspdNCG.n1  = n;
optspdNCG.n2  = n;
optspdNCG.b   = f;
optspdNCG.A   = ApdNCG;
optspdNCG.tau = 0.1;
optspdNCG.tol = 1.0e-1;
optspdNCG.problem  = 'tv';
optspdNCG.verbose  = 2;

tstart = cputime;
[xDC, dual, out] = pdNCGinterface(optspdNCG);
time_pdNCG = cputime - tstart;

subplot(133); 
mat = @(x) reshape(x,n,n);
imshow(mat(xDC),[]);
title('Recovered by TVAL3','fontsize',18);
xlabel(sprintf(' %2d%% measurements \n Rel-Err: %4.2f%%, CPU: %4.2fs ',ratio*100,norm(mat(xDC)-I,'fro')/nrmI*100,time_pdNCG),'fontsize',16);