%DEMOTV Demonstrates the usage of the pdNCG solver for real
%TV-regularization image reconstruction problems:
%  
%  min tau*||x||_TV + 0.5*||Ax - b||^2.
%
% This example has been obtained from http://dsp.rice.edu/cscamera.

clear;

fprintf('\nExample TV-regularization \n \n');

% Image of size n x n.
n = 64;

% Load measurements - Column Vector
eval(['load ' 'Ball_64']);

% Image size
N = n^2;

% Options.
opts.n1   = n;
opts.n2   = n;
opts.tau  = 5.0e-3;
opts.problem = 'tv';

figure(1); clf;
    
for i=1:6
    % Ratio of measurements Measurements, initialized to 10%.
    subrate = 0.1*i;

    % Get desired number of samples
    b = y(1:(round(N*subrate)));

    % Number of measurements
    M = length(b);

    % Load measurement matrix and truncate
    % Each row of Phi contains a measurement vector of length N
    % and is a raster scan of the 0/1 pattern fed to the DMD
    % see http://dsp.rice.edu/cscamera.
    eval(['load Phi_' num2str(n)])
    Phi = Phi(1:M,:);

    % Options.
    opts.b = b*subrate;
    opts.A = Phi;

    % Call pdNCG.
    [x, dual, out] = pdNCGinterface(opts);

    % Reshape output; flip upside down
    x = reshape(x,sqrt(N),sqrt(N));
    x = flipud(x);

    % Normalize output
    x = (x-min(min(x)))/(max(max(x))-min(min(x)));

    % Show reconstruction
    subplot(2,3,i); imagesc(x),colormap(gray),axis image
    title([num2str(i*10),'% of measurements']);
    axis off;
end

clear;
