%DEMOTVL1ANALYSIS Demonstrates the usage of the pdNCG solver for real 
% TV-regularization combined with L1-Analysis image reconstruction 
% problems:
%  
%      min tau_TV*||x||_TV + tau_W*||Wx||_1 + 0.5*||Ax - b||^2.
%
% The problem instance that is used has been copied from 
% TFOCS package, url: http://cvxr.com/tfocs/download/.
%
%  ** Requires Wavelet Toolbox **

clear;

fprintf('Example TV-regularization combined with L1-Analysis, ** Requires Wavelet Toolbox ** \n \n');

% Select phantom image
x_opt = phantom(256);
[n1,n2] = size(x_opt);
N = n1*n2;
        
% Defined vec and mat functions to reshape objects.
mat = @(x) reshape(x,n1,n2);
vec = @(x) x(:);

% The measurement is the identity, i.e. denoising.
A = A_operator(@(x) x, @(x) x); 

% Noise function.
awgn = @(x,snr) x + 10^( (10*log10(sum(abs(x(:)).^2)/length(x(:))) - snr)/20 )*randn(size(x));
psnr = @(x) 20*log10(sqrt(N)/norm(vec(x)-vec(x_opt)));

snr = 5;  % SNR in dB.

% Calculate measurements.
b = A*vec(x_opt);

% Add noise.
x_noisy = awgn( x_opt, snr );

% Calculate noisy measurements.
b_n   = A*vec(x_noisy);

% L1-analysis operator.
% symmetric 9/7 biothogonal wavelets are what JPEG-2000 uses
waveletF = 'bior4.4';
waveletT = 'rbio4.4';
levels = min( 1, wmaxlev(size(x_noisy),waveletF) );
[c,s] = wavedec2(x_noisy,levels,waveletF); 

W = A_operator(@(q) vec(waverec2(vec(q),s,waveletT) ), ...
    @(q) wavedec2(mat(q),levels,waveletF)');

% Options.
opts.n1      = n1;
opts.n2      = n2;
opts.b       = b_n;
opts.A       = A;
opts.W       = W;
opts.tol     = 1.0e+0;
opts.tau_TV  = 3.0e-1;
opts.tau_W   = 3.0e-2;
opts.problem = 'tv_wavelet';

% Call pdNCG.
[x, y, out] = pdNCGinterface(opts);

% Show reconstruction
subplot(1,3,1); imshow(x_opt);
title(sprintf('Original, PSNR %.1f dB', psnr(x_opt)));
subplot(1,3,2); imshow(x_noisy);
title(sprintf('Noisy, PSNR %.1f dB', psnr(x_noisy)));
subplot(1,3,3); imshow(mat(x));
title(sprintf('Reconstructed, PSNR %.1f dB', psnr(x)));

clear;
